#include "StdAfx.h"
#include "MainDialog.h"
#include "resource.h"
#include "misc.h"
#include "AboutDlg.h"
#include <windowsx.h>
#include <objbase.h>
#include <gdiplus.h>

#define IDM_SYS_ABOUT 0x10


MainDialog::MainDialog(UINT idResource)
	: Dialog( idResource )
	, timerID( 0 )
{
	Gdiplus::GdiplusStartupInput gdiSI;
	GdiplusStartup( &gdiToken, &gdiSI, NULL );
}

MainDialog::~MainDialog(void)
{
	Gdiplus::GdiplusShutdown( gdiToken );
	UnregisterClass( szWindowClass, hInst );
}

INT_PTR CALLBACK MainDialog::DlgProc( UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	switch( uMsg )
	{
	HANDLE_MSG( m_hDlg, WM_INITDIALOG, Cls_OnInitDialog );
	HANDLE_MSG( m_hDlg, WM_SYSCOMMAND, Cls_OnSysCommand );
	HANDLE_MSG( m_hDlg, WM_COMMAND, Cls_OnCommand );
	HANDLE_MSG( m_hDlg, WM_DESTROY, Cls_OnDestroy );
	HANDLE_MSG( m_hDlg, WM_TIMER, Cls_OnTimer );

	default:
		return DefWindowProc(m_hDlg, uMsg, wParam, lParam);
		break;
	}

	return 0;
}

BOOL MainDialog::InitInstance( HWND hParent )
{
	WNDCLASSEX wcex;

	wcex.cbSize = sizeof(WNDCLASSEX);

	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= (WNDPROC)_RootWndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= DLGWINDOWEXTRA;
	wcex.hInstance		= hInst;
	wcex.hIcon			= LoadIcon(hInst, MAKEINTRESOURCE(IDI_DIALOG));
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_BTNFACE+1);
	wcex.lpszMenuName	= NULL;
	wcex.lpszClassName	= szWindowClass;
	wcex.hIconSm		= NULL;

	return RegisterClassEx(&wcex);
}

BOOL MainDialog::DoModeless(HWND hParent, int nCmdShow)
{
	BOOL bRet = FALSE;

	if ( InitInstance() ) {
		if ( bRet = Dialog::DoModeless( hParent, nCmdShow ) ) {
			SetWindowText( m_hDlg, szTitle );
		}
	}
	return bRet;
}

void MainDialog::InsertSystemMenu( void )
{
	MENUITEMINFO mii;
	HMENU hSysMenu;
	TCHAR szTemp[256];
	int nMenuItemCount;

	memset ( &mii, 0, sizeof(MENUITEMINFO) );
	mii.cbSize = sizeof(MENUITEMINFO);

	mii.fMask = MIIM_TYPE;
	mii.fType = MFT_SEPARATOR;

	// Zp[^ǉ
	hSysMenu = GetSystemMenu( m_hDlg, FALSE );
	nMenuItemCount = GetMenuItemCount ( hSysMenu );
	InsertMenuItem ( hSysMenu, nMenuItemCount -1, TRUE, &mii );

	LoadString( hInst, IDS_ABOUTBOX, szTemp, 256 );
	mii.fMask	   = MIIM_ID | MIIM_TYPE;
	mii.fType	   = MFT_STRING;
	mii.wID 	   = IDM_SYS_ABOUT;
	mii.dwTypeData = szTemp;
	mii.cch 	   = _tcslen(szTemp);

	InsertMenuItem( hSysMenu, nMenuItemCount -1, TRUE, &mii );
	DrawMenuBar( m_hDlg );
}

BOOL MainDialog::Cls_OnInitDialog(HWND hwnd, HWND hwndFocus, LPARAM lParam)
{
	InsertSystemMenu();
	CoInitialize(NULL);
	taskbar.Init();

	SendDlgItemMessage( hwnd, IDC_SLIDER_PROGRESS, TBM_SETRANGEMIN, TRUE,   0 );
	SendDlgItemMessage( hwnd, IDC_SLIDER_PROGRESS, TBM_SETRANGEMAX, TRUE, 100 );
	SetCheck( IDC_RADIO_NONE, BST_CHECKED );

	return TRUE;
}

void MainDialog::Cls_OnSysCommand(HWND hwnd, UINT cmd, int x, int y)
{
	int wmId;
	AboutDlg *aboutDlg = NULL;

	wmId	= LOWORD(cmd) & 0xfff0;
	switch(wmId)
	{
	case IDM_SYS_ABOUT:
		if ( aboutDlg ) {
			aboutDlg->SetFocus();
		}
		else {
			aboutDlg = new AboutDlg(IDD_ABOUTBOX);
			aboutDlg->DoModal(hwnd);
			delete aboutDlg;
			aboutDlg = NULL;
		}
		break;
	default:
		DefWindowProc(hwnd, WM_SYSCOMMAND, cmd, MAKELPARAM( x, y ) );
	}
}

void MainDialog::Cls_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
	switch( id )
	{
	case IDOK:
	case IDCANCEL:
		DestroyWindow( m_hDlg );
		break;

	case IDC_BTN_APPLY:
		Cmd_OnBtnBadge();
		break;

	case IDC_BTN_PROGRESS:
		Cmd_OnBtnProgress();
		break;

	case IDC_BTN_FLASH:
		Cmd_OnBtnFlash();
		break;
	}
}

void MainDialog::Cls_OnDestroy(HWND hwnd)
{
	PostQuitMessage( 0 );
}

void MainDialog::Cls_OnTimer(HWND hwnd, UINT id)
{
	KillTimer( m_hDlg, timerID );
	timerID = 0;
	taskbar.Flash( m_hDlg );
}

void MainDialog::Cmd_OnBtnBadge(void)
{
	TCHAR str[10];
	Taskbar3::BADGE_SIGNTYPE type = Taskbar3::SIGN_NONE;

	GetItemText( IDC_EDIT_STRING, str, _countof(str) );

	if ( GetCheck( IDC_RADIO_NONE  ) ) type = Taskbar3::SIGN_NONE;
	if ( GetCheck( IDC_RADIO_PLUS  ) ) type = Taskbar3::SIGN_PLUS;
	if ( GetCheck( IDC_RADIO_MINUS ) ) type = Taskbar3::SIGN_MINUS;

	taskbar.SetBadge( m_hDlg, str, type );
}

void MainDialog::Cmd_OnBtnProgress(void)
{
	DWORD dwProgress = (DWORD)SendDlgItemMessage( m_hDlg, IDC_SLIDER_PROGRESS, TBM_GETPOS, 0, 0 );
	taskbar.SetProgress( m_hDlg, dwProgress, 100 );
}

void MainDialog::Cmd_OnBtnFlash(void)
{
	if ( timerID == 0 ) timerID = SetTimer( m_hDlg, 1, 5000, NULL );
}
