#include "StdAfx.h"
#include "Taskbar3.h"

using namespace Gdiplus;


Taskbar3::Taskbar3(void)
{
}

Taskbar3::~Taskbar3(void)
{
}

BOOL Taskbar3::Init(void)
{
	return SUCCEEDED( taskBar.CreateInstance(CLSID_TaskbarList) );
}

void Taskbar3::AddSign( Gdiplus::Graphics &gr, Gdiplus::FontFamily &ff, Gdiplus::StringFormat &format, BADGE_ADDSIGNTYPE addType )
{
	if ( addType != ADD_NONE ) {

		INT style = 0;
		RectF textRect;
		GraphicsPath gp;
		Matrix moveToCenter;
		REAL signx, signy;
		TCHAR *signStr = ( addType == ADD_PLUS_UR ) ? _T("+") : _T("-");

		Pen redPen( Color( 255, 255, 0, 0 ), 1.5 );
		SolidBrush whiteBrush( Color( 255, 255, 255, 255 ) );

		switch( addType )
		{
		case ADD_MINUS_UR:
			style += FontStyleBold;
		case ADD_PLUS_UR:
			signx = 13;
			signy = 5;
			break;
		case ADD_MINUS_LEFT:
			signx = 3;
			signy = 8;
			break;

		default:
			return;
		}
		gp.Reset();
		gp.AddString( signStr, -1, &ff, style, 14, PointF( 0, 0 ), &format );
		moveToCenter.Reset();
		gp.GetBounds( &textRect );
		moveToCenter.Translate( (REAL)signx - textRect.Width / (REAL)2 - textRect.X, (REAL)signy - textRect.Height / (REAL)2 - textRect.Y );
		gp.Transform( &moveToCenter );

		gr.DrawPath( &redPen, &gp );
		gr.FillPath( &whiteBrush, &gp );
	}

}

HICON Taskbar3::GenerateBadgeIcon( TCHAR *str, BADGE_SIGNTYPE type )
{
	HDC hMemDC;
	DWORD dwWidth, dwHeight;
	BITMAPV5HEADER bi;
	HBITMAP hBitmap, hOldBitmap;
	void *lpBits;
	HCURSOR hAlphaCursor = NULL;
	BOOL minusLeft = FALSE;

	dwWidth  = 16;  // width of cursor
	dwHeight = 16;  // height of cursor

	ZeroMemory(&bi,sizeof(BITMAPV5HEADER));
	bi.bV5Size		   = sizeof(BITMAPV5HEADER);
	bi.bV5Width		   = dwWidth;
	bi.bV5Height		  = dwHeight;
	bi.bV5Planes = 1;
	bi.bV5BitCount = 32;
	bi.bV5Compression = BI_BITFIELDS;
	// The following mask specification specifies a supported 32 BPP
	// alpha format for Windows XP.
	bi.bV5RedMask   =  0x00FF0000;
	bi.bV5GreenMask =  0x0000FF00;
	bi.bV5BlueMask  =  0x000000FF;
	bi.bV5AlphaMask =  0xFF000000;

	HDC hdc;
	hdc = GetDC(NULL);

	// Create the DIB section with an alpha channel.
	hBitmap = CreateDIBSection(hdc, (BITMAPINFO *)&bi, DIB_RGB_COLORS,
		(void **)&lpBits, NULL, (DWORD)0);

	hMemDC = CreateCompatibleDC(hdc);
	ReleaseDC(NULL,hdc);

	// Draw something on the DIB section.
	hOldBitmap = (HBITMAP)SelectObject(hMemDC, hBitmap);

	Graphics gr(hMemDC);
	gr.SetSmoothingMode( SmoothingModeHighQuality );

	SolidBrush backBrush( Color( 0, 0, 0, 0 ) );
	gr.FillRectangle( &backBrush, RectF( 0, 0, 16, 16 ) );

	SolidBrush whiteBrush( Color( 255, 255, 255, 255 ) );
	gr.FillEllipse( &whiteBrush, 0, 0, 16, 16 );

	SolidBrush redBrush( Color( 255, 255, 0, 0 ) );
	gr.FillEllipse( &redBrush, RectF( 1.5, 1.5, 13, 13 ) );

	FontFamily ff( _T("Franklin Gothic Medium") );
	StringFormat format;
	GraphicsPath gp;

	if ( ( _tcslen(str) > 1 ) && str[0] == _T('-') ) {
		str++;
		minusLeft = TRUE;
	}
	format.SetAlignment( StringAlignmentCenter );
	gp.AddString( str, -1, &ff, 0, 14, PointF( 0, 0 ), &format );

	RectF textRect;
	gp.GetBounds( &textRect );

	if ( ( textRect.Width > 12.0 ) || ( textRect.Height > 12.0 ) ) {
		Matrix reduceTo12;

		REAL scalex, scaley;
		if ( _tcslen(str) == 1 ) {
			scalex = scaley = (REAL)12.0 / max( textRect.Width, textRect.Height );
		}
		else {
			scalex = (REAL)10.0 / textRect.Width;
			scaley = (REAL)10.0 / textRect.Height;
		}

		reduceTo12.Scale( scalex, scaley );
		gp.Transform( &reduceTo12 );
		gp.GetBounds( &textRect );
	}

	Matrix moveToCenter;
	moveToCenter.Translate( (REAL)8, ( (REAL)16 - textRect.Height ) / (REAL)2 - textRect.Y );
	gp.Transform( &moveToCenter );

	gr.FillPath( &whiteBrush, &gp );

	if ( minusLeft ) AddSign( gr, ff, format, ADD_MINUS_LEFT );
	if ( type != SIGN_NONE ) AddSign( gr, ff, format, (BADGE_ADDSIGNTYPE)type );

	SelectObject(hMemDC, hOldBitmap);
	DeleteDC(hMemDC);

	// Create an empty mask bitmap.
	HBITMAP hMonoBitmap = CreateBitmap(dwWidth,dwHeight,1,1,NULL);

	ICONINFO ii;
	ii.fIcon = FALSE;  // Change fIcon to TRUE to create an alpha icon
	ii.xHotspot = 0;
	ii.yHotspot = 0;
	ii.hbmMask = hMonoBitmap;
	ii.hbmColor = hBitmap;

	// Create the alpha cursor with the alpha DIB section.
	hAlphaCursor = CreateIconIndirect(&ii);

	DeleteObject(hBitmap);
	DeleteObject(hMonoBitmap);

	return hAlphaCursor;
}

BOOL Taskbar3::SetBadge( HWND hWnd, TCHAR *str, BADGE_SIGNTYPE type )
{
	BOOL bRet = FALSE;
	HRESULT hr;

	do {
		if ( taskBar == NULL ) break;

		if ( str && ( str[0] != 0 ) ) {
			HICON icon = GenerateBadgeIcon( str, type );
			hr = taskBar->SetOverlayIcon( hWnd, icon, str );
			DeleteObject( icon );
		}
		else {
			hr = taskBar->SetOverlayIcon( hWnd, NULL, _T("") );
		}
		if ( SUCCEEDED(hr) ) bRet = TRUE;

	} while( FALSE );

	return bRet;
}

BOOL Taskbar3::SetProgress( HWND hWnd, ULONGLONG ullCompleted, ULONGLONG ullTotal )
{
	BOOL bRet = FALSE;

	do {
		if ( taskBar == NULL ) break;

		if ( ullCompleted > 0 ) {
			if ( FAILED( taskBar->SetProgressValue( hWnd, ullCompleted, ullTotal ) ) ) break;
		}
		else {
			if ( FAILED( taskBar->SetProgressState( hWnd, TBPF_NOPROGRESS ) ) ) break;
		}

		bRet = TRUE;

	} while( FALSE );

	return bRet;
}

BOOL Taskbar3::Flash( HWND hWnd )
{
	BOOL bRet = FALSE;

	do {
		FLASHWINFO fwi;

		fwi.cbSize = sizeof(FLASHWINFO);
		fwi.hwnd = hWnd;
		fwi.dwFlags = FLASHW_ALL | FLASHW_TIMERNOFG;
		fwi.uCount = 0;
		fwi.dwTimeout = 0;
		FlashWindowEx( &fwi );

		if ( taskBar == NULL ) break;

		if ( FAILED( taskBar->SetProgressState( hWnd, TBPF_NOPROGRESS ) ) ) break;

		bRet = TRUE;

	} while( FALSE );

	return bRet;
}
